@@ -1,7 +1,7 @@
 /*                      _             _
 **  _ __ ___   ___   __| |    ___ ___| |
 ** | '_ ` _ \ / _ \ / _` |   / __/ __| |
-** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to SSLeay
+** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to OpenSSL
 ** |_| |_| |_|\___/ \__,_|___|___/___/_| http://www.engelschall.com/sw/mod_ssl/
 **                      |_____|
 **  mod_ssl.c
@@ -198,7 +198,7 @@ module MODULE_VAR_EXPORT ssl_module = {
     ssl_hook_AddModule,       /* after modules was added to core     */
     ssl_hook_RemoveModule,    /* before module is removed from core  */
     ssl_hook_RewriteCommand,  /* configuration command rewriting     */
-    ssl_hook_NewConnection    /* configuration command rewriting     */
+    ssl_hook_NewConnection,   /* socket connection open              */
+    ssl_hook_CloseConnection  /* socket connection close             */
 };
 
-
