@@ -1,7 +1,7 @@
 /*                      _             _
 **  _ __ ___   ___   __| |    ___ ___| |
 ** | '_ ` _ \ / _ \ / _` |   / __/ __| |
-** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to SSLeay
+** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to OpenSSL
 ** |_| |_| |_|\___/ \__,_|___|___/___/_| http://www.engelschall.com/sw/mod_ssl/
 **                      |_____|
 **  mod_ssl.h
@@ -77,7 +77,7 @@
 #include <errno.h>
 #include <sys/stat.h>
 
-/* SSLeay headers */
+/* OpenSSL headers */
 #include <ssl.h>
 #include <err.h>
 #include <x509.h>
@@ -119,6 +119,10 @@
 #ifndef NUL
 #define NUL   '\0'
 #endif
+#ifndef RAND_MAX
+#include <limits.h>
+#define RAND_MAX INT_MAX
+#endif
 
 /*
  * Provide reasonable defines for some types
@@ -148,6 +152,7 @@
 #define cfgMerge(el,unset)  new->el = add->el == unset ? base->el : add->el
 #define cfgMergeArray(el)   new->el = ap_append_arrays(p, add->el, base->el)
 #define cfgMergeTable(el)   new->el = ap_overlay_tables(p, add->el, base->el)
+#define cfgMergeCtx(el)     new->el = ap_ctx_overlay(p, add->el, base->el)
 #define cfgMergeString(el)  cfgMerge(el, NULL)
 #define cfgMergeBool(el)    cfgMerge(el, UNSET)
 #define cfgMergeInt(el)     cfgMerge(el, UNSET)
@@ -285,11 +290,11 @@
 #endif
 
 /*
- * Check for SSLeay 0.9.0 and below
+ * Check for SSLeay/OpenSSL 0.9.0 and below
  * and enabled Extended API (EAPI)
  */
 #if SSL_LIBRARY_VERSION < 0x0900
-#error "SSLeay versions below 0.9.0 are no longer supported"
+#error "SSLeay/OpenSSL versions below 0.9.0 are no longer supported"
 #endif
 #ifndef EAPI
 #error "mod_ssl requires Extended API (EAPI)"
@@ -433,12 +438,7 @@ typedef struct {
 typedef struct {
     pool           *pPool;
     BOOL            bFixed;
-
-    /*
-     * global config data
-     */
     int             nInitCount;
-
     RSA            *pRSATmpKey;
     int             nSessionCacheMode;
     char           *szSessionCacheDataFile;
@@ -447,16 +447,14 @@ typedef struct {
     int             nMutexFD;
     int             nMutexSEMID;
     array_header   *aRandSeed;
-
     ssl_ds_table   *tPublicCert;
     ssl_ds_table   *tPrivateKey;
-
-    /*
-     * arbitrary global context data
-     */
     struct {
         void *pV1, *pV2, *pV3, *pV4, *pV5, *pV6, *pV7, *pV8, *pV9;
     } rCtx;
+#ifdef SSL_VENDOR
+    ap_ctx         *ctx;
+#endif
 } SSLModConfigRec;
 
 /*
@@ -483,6 +481,9 @@ typedef struct {
     int          nPassPhraseDialogType;
     char        *szPassPhraseDialogPath;
     ssl_proto_t  nProtocol;
+#ifdef SSL_VENDOR
+    ap_ctx      *ctx;
+#endif
 } SSLSrvConfigRec;
 
 /*
@@ -503,6 +504,9 @@ typedef struct {
     char         *szCACertificatePath;
     char         *szCACertificateFile;
 #endif
+#ifdef SSL_VENDOR
+    ap_ctx       *ctx;
+#endif
 } SSLDirConfigRec;
 
 /*
@@ -553,15 +557,19 @@ void         ssl_hook_RemoveModule(module *);
 char        *ssl_hook_RewriteCommand(cmd_parms *, void *config, const char *);
 void         ssl_hook_NewConnection(conn_rec *);
 void         ssl_hook_TimeoutConnection(int);
-void         ssl_hook_CloseConnection(void *);
+void         ssl_hook_CloseConnection(conn_rec *);
 int          ssl_hook_Auth(request_rec *);
 int          ssl_hook_Access(request_rec *);
 int          ssl_hook_Fixup(request_rec *);
 int          ssl_hook_ReadReq(request_rec *);
 int          ssl_hook_Handler(request_rec *);
 
-/*  SSLeay callbacks */
+/*  OpenSSL callbacks */
+#if SSL_LIBRARY_VERSION >= 0x0920
+RSA         *ssl_callback_TmpRSA(SSL *, int, int);
+#else
 RSA         *ssl_callback_TmpRSA(SSL *, int);
+#endif
 int          ssl_callback_SSLVerify(int, X509_STORE_CTX *);
 int          ssl_callback_NewSessionCacheEntry(SSL *, SSL_SESSION *);
 SSL_SESSION *ssl_callback_GetSessionCacheEntry(SSL *, unsigned char *, int, int *);
@@ -656,4 +664,10 @@ BOOL         ssl_util_path_check(ssl_pathcheck_t, char *);
 char        *ssl_util_ptxtsub(pool *, const char *, const char *, char *);
 void         ssl_util_thread_setup(void);
 
+/*  Vendor extension support  */
+#if defined(SSL_VENDOR) && defined(SSL_VENDOR_OBJS)
+void         ssl_vendor_register(void);
+void         ssl_vendor_unregister(void);
+#endif
+
 #endif /* MOD_SSL_H */
