@@ -1,7 +1,7 @@
 /*                      _             _
 **  _ __ ___   ___   __| |    ___ ___| |
 ** | '_ ` _ \ / _ \ / _` |   / __/ __| |
-** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to SSLeay
+** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to OpenSSL
 ** |_| |_| |_|\___/ \__,_|___|___/___/_| http://www.engelschall.com/sw/mod_ssl/
 **                      |_____|
 **  ssl_engine_compat.c
@@ -78,9 +78,11 @@
  * The mapping of obsolete directives to official ones...
  */
 
-static char *ssl_compat_SSLRequireCipher(pool *p, const char *, const char *, const char *);
-static char *ssl_compat_SSLBanCipher(pool *p, const char *, const char *, const char *);
-static char *ssl_compat_words2list(pool *p, const char *);
+static char *ssl_compat_SSLSessionLockFile(pool *, const char *, const char *, const char *);
+static char *ssl_compat_SSLCacheDisable(pool *, const char *, const char *, const char *);
+static char *ssl_compat_SSLRequireCipher(pool *, const char *, const char *, const char *);
+static char *ssl_compat_SSLBanCipher(pool *, const char *, const char *, const char *);
+static char *ssl_compat_words2list(pool *, const char *);
 
 #define CRM_BEGIN              /* nop */
 #define CRM_ENTRY(what,action) { what, action },
@@ -143,8 +145,13 @@ static struct {
     /*
      * Stronghold 2.x backward compatibility
      */
+    CRM_ENTRY( CRM_CMD("StrongholdAccelerator"),     CRM_LOG("Not supported by mod_ssl")      )
+    CRM_ENTRY( CRM_CMD("StrongholdKey"),             CRM_LOG("Not supported by mod_ssl")      )
+    CRM_ENTRY( CRM_CMD("StrongholdLicenseFile"),     CRM_LOG("Not supported by mod_ssl")      )
     CRM_ENTRY( CRM_CMD("SSLFlag"),                   CRM_SUB("SSLEngine")                     )
-    CRM_ENTRY( CRM_CMD("SSLSessionLockFile"),        CRM_SUB("SSLMutex")                      )
+    CRM_ENTRY( CRM_CMD("SSLClientCAfile"),           CRM_SUB("SSLCACertificateFile")          )
+    CRM_ENTRY( CRM_CMD("SSLSessionLockFile"),        CRM_CAL(ssl_compat_SSLSessionLockFile)   )
+    CRM_ENTRY( CRM_CMD("SSLCacheDisable"),           CRM_CAL(ssl_compat_SSLCacheDisable)      )
     CRM_ENTRY( CRM_CMD("RequireSSL"),                CRM_SUB("SSLRequireSSL")                 )
     CRM_ENTRY( CRM_CMD("SSLCipherList"),             CRM_SUB("SSLCipherSuite")                )
     CRM_ENTRY( CRM_CMD("SSLErrorFile"),              CRM_LOG("Not needed for mod_ssl")        )
@@ -162,6 +169,28 @@ static struct {
     CRM_END
 };
 
+static char *ssl_compat_SSLSessionLockFile(
+    pool *p, const char *oline, const char *cmd, const char *args)
+{
+    char *cp;
+
+    for (cp = (char *)args; ap_isspace(*cp); cp++)
+        ;
+    return ap_pstrcat(p, "SSLMutex file:", cp, NULL);
+}
+
+static char *ssl_compat_SSLCacheDisable(
+    pool *p, const char *oline, const char *cmd, const char *args)
+{
+    char *cp;
+
+    for (cp = (char *)args; ap_isspace(*cp); cp++)
+        ;
+    if (strcEQ(cp, "on"))
+        return "SSLSessionCache none";
+    return "";
+}
+
 static char *ssl_compat_SSLRequireCipher(pool *p, const char *oline, const char *cmd, const char *args)
 {
     return ap_pstrcat(p, "SSLRequire %{SSL_CIPHER} in {",
@@ -382,8 +411,15 @@ static struct {
     VRM_ENTRY( VRM_VAR("SSL_CLIENT_IC"),                 VRM_SUB("SSL_CLIENT_I_DN_C")        )
     VRM_ENTRY( VRM_VAR("SSL_CLIENT_ISP"),                VRM_SUB("SSL_CLIENT_I_DN_SP")       )
     VRM_ENTRY( VRM_VAR("SSL_CLIENT_IL"),                 VRM_SUB("SSL_CLIENT_I_DN_L")        )
+    VRM_ENTRY( VRM_VAR("SSL_EXPORT"),                    VRM_SUB("SSL_CIPHER_EXPORT")        )
+    VRM_ENTRY( VRM_VAR("SSL_KEYSIZE"),                   VRM_SUB("SSL_CIPHER_ALGKEYSIZE")    )
+    VRM_ENTRY( VRM_VAR("SSL_SECRETKEYSIZE"),             VRM_SUB("SSL_CIPHER_USEKEYSIZE")    )
+    VRM_ENTRY( VRM_VAR("SSL_SSLEAY_VERSION"),            VRM_SUB("SSL_VERSION_LIBRARY")      )
 
+    VRM_ENTRY( VRM_VAR("SSL_SERVER_CERT_SERIAL"),        VRM_LOG("Not supported by mod_ssl") )
+    VRM_ENTRY( VRM_VAR("SSL_STRONG_CRYPTO"),             VRM_LOG("Not supported by mod_ssl") )
     VRM_ENTRY( VRM_VAR("SSL_SERVER_KEY_EXP"),            VRM_LOG("Not supported by mod_ssl") )
+    VRM_ENTRY( VRM_VAR("SSL_SERVER_KEY_SIZE"),           VRM_LOG("Not supported by mod_ssl") )
     VRM_ENTRY( VRM_VAR("SSL_SERVER_KEY_ALGORITHM"),      VRM_LOG("Not supported by mod_ssl") )
     VRM_ENTRY( VRM_VAR("SSL_SERVER_SIGNATURE_ALGORITHM"),VRM_LOG("Not supported by mod_ssl") )
     VRM_ENTRY( VRM_VAR("SSL_SERVER_SESSIONDIR"),         VRM_LOG("Not supported by mod_ssl") )
@@ -417,9 +453,25 @@ void ssl_compat_variables(request_rec *r)
                 ap_table_set(r->subprocess_env, cpOld, cpVal);
         }
         else if (cpMsg != NULL) {
+#ifdef SSL_VENDOR
+           /* 
+            * something that isn't provided by mod_ssl, so at least
+            * let vendor extensions provide a reasonable value first.
+            */
+            cpVal = NULL;
+            ap_hook_use("ssl::vendor::compat_variables_lookup", 
+                        AP_HOOK_SIG3(ptr,ptr,ptr), 
+                        AP_HOOK_DECLINE(NULL), 
+                        &cpVal, r, cpOld);
+            if (cpVal != NULL) {
+                ap_table_set(r->subprocess_env, cpOld, cpVal);
+                continue;
+            }
+#endif
+
             /*
-             * we cannot print a message, so we at least the
-             * variables contents to the message
+             * we cannot print a message, so we set at least 
+             * the variables content to the compat message
              */
             ap_table_set(r->subprocess_env, cpOld, cpMsg);
         }
