@@ -1,7 +1,7 @@
 /*                      _             _
 **  _ __ ___   ___   __| |    ___ ___| |
 ** | '_ ` _ \ / _ \ / _` |   / __/ __| |
-** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to SSLeay
+** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to OpenSSL
 ** |_| |_| |_|\___/ \__,_|___|___/___/_| http://www.engelschall.com/sw/mod_ssl/
 **                      |_____|
 **  ssl_engine_config.c
@@ -85,6 +85,9 @@ void ssl_hook_AddModule(module *m)
         ssl_var_register();
         ssl_ext_register();
         ssl_io_register();
+#if defined(SSL_VENDOR) && defined(SSL_VENDOR_OBJS)
+        ssl_vendor_register();
+#endif
     }
     return;
 }
@@ -98,6 +101,9 @@ void ssl_hook_RemoveModule(module *m)
         ssl_var_unregister();
         ssl_ext_unregister();
         ssl_io_unregister();
+#if defined(SSL_VENDOR) && defined(SSL_VENDOR_OBJS)
+        ssl_vendor_unregister();
+#endif
     }
     return;
 }
@@ -129,10 +135,15 @@ void ssl_config_global_create(void)
         mc->nMutexFD               = -1;
         mc->nMutexSEMID            = -1;
         mc->aRandSeed              = ap_make_array(pPool, 4, sizeof(ssl_randseed_t));
-
         mc->tPrivateKey            = ssl_ds_table_make(pPool, sizeof(ssl_asn1_t));
         mc->tPublicCert            = ssl_ds_table_make(pPool, sizeof(ssl_asn1_t));
 
+#ifdef SSL_VENDOR
+        mc->ctx = ap_ctx_new(pPool);
+        ap_hook_use("ssl::vendor::config_global_create", 
+                AP_HOOK_SIG2(void,ptr), AP_HOOK_MODE_ALL, mc);
+#endif
+
         /*
          * And push it into Apache's global context
          */
@@ -190,6 +201,13 @@ void *ssl_config_server_create(pool *p, server_rec *s)
     sc->prsaKey                = NULL;
     sc->pSSLCtx                = NULL;
 
+#ifdef SSL_VENDOR
+    sc->ctx = ap_ctx_new(p);
+    ap_hook_use("ssl::vendor::config_server_create", 
+                AP_HOOK_SIG4(void,ptr,ptr,ptr), AP_HOOK_MODE_ALL,
+                p, s, sc);
+#endif
+
     return sc;
 }
 
@@ -221,6 +239,13 @@ void *ssl_config_server_merge(pool *p, void *basev, void *addv)
     cfgMerge(prsaKey, NULL);
     cfgMerge(pSSLCtx, NULL);
 
+#ifdef SSL_VENDOR
+    cfgMergeCtx(ctx);
+    ap_hook_use("ssl::vendor::config_server_merge", 
+                AP_HOOK_SIG5(void,ptr,ptr,ptr,ptr), AP_HOOK_MODE_ALL,
+                p, base, add, new);
+#endif
+
     return new;
 }
 
@@ -245,6 +270,13 @@ void *ssl_config_perdir_create(pool *p, char *dir)
     dc->szCACertificateFile    = NULL;
 #endif
 
+#ifdef SSL_VENDOR
+    dc->ctx = ap_ctx_new(p);
+    ap_hook_use("ssl::vendor::config_perdir_create", 
+                AP_HOOK_SIG4(void,ptr,ptr,ptr), AP_HOOK_MODE_ALL,
+                p, dir, dc);
+#endif
+
     return dc;
 }
 
@@ -280,6 +312,13 @@ void *ssl_config_perdir_merge(pool *p, void *basev, void *addv)
     cfgMergeString(szCACertificateFile);
 #endif
 
+#ifdef SSL_VENDOR
+    cfgMergeCtx(ctx);
+    ap_hook_use("ssl::vendor::config_perdir_merge", 
+                AP_HOOK_SIG5(void,ptr,ptr,ptr,ptr), AP_HOOK_MODE_ALL,
+                p, base, add, new);
+#endif
+
     return new;
 }
 
@@ -556,7 +595,12 @@ const char *ssl_cmd_SSLSessionCache(
         mc->szSessionCacheDataFile = ap_pstrdup(mc->pPool,
                                      ap_server_root_relative(cmd->pool, arg+4));
     }
-    else
+	else
+#ifdef SSL_VENDOR
+        if (!ap_hook_use("ssl::vendor::cmd_sslsessioncache", 
+             AP_HOOK_SIG4(void,ptr,ptr,ptr), AP_HOOK_MODE_ALL, 
+             cmd, arg, mc))
+#endif
         return "SSLSessionCache: Invalid argument";
     return NULL;
 }
