@@ -1,7 +1,7 @@
 /*                      _             _
 **  _ __ ___   ___   __| |    ___ ___| |
 ** | '_ ` _ \ / _ \ / _` |   / __/ __| |
-** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to SSLeay
+** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to OpenSSL
 ** |_| |_| |_|\___/ \__,_|___|___/___/_| http://www.engelschall.com/sw/mod_ssl/
 **                      |_____|
 **  ssl_engine_kernel.c
@@ -177,12 +177,13 @@ void ssl_hook_NewConnection(conn_rec *conn)
     SSL_set_app_data2(ssl, NULL); /* request_rec (available later)   */
     SSL_set_fd(ssl, fb->fd);
     ap_ctx_set(fb->ctx, "ssl", ssl);
-    ap_register_cleanup(conn->pool, (void *)conn,
-                        ssl_hook_CloseConnection, ssl_hook_CloseConnection);
 
     /*
-     * Configure SSLeay BIO Data Logging
+     *  Configure callbacks for SSL connection
      */
+#if SSL_LIBRARY_VERSION >= 0x0920
+    SSL_set_tmp_rsa_callback(ssl, ssl_callback_TmpRSA);
+#endif
     if (sc->nLogLevel >= SSL_LOG_DEBUG) {
         BIO_set_callback(SSL_get_rbio(ssl), ssl_io_data_cb);
         BIO_set_callback_arg(SSL_get_rbio(ssl), ssl);
@@ -192,25 +193,23 @@ void ssl_hook_NewConnection(conn_rec *conn)
      * Configure the server certificate and private key
      * which should be used for this connection.
      */
-    if (sc->szCertificateFile != NULL) {
-        if (SSL_use_certificate(ssl, sc->px509Certificate) <= 0) {
-            ssl_log(conn->server, SSL_LOG_ERROR|SSL_ADD_SSLERR,
-                    "Unable to configure server certificate for connection");
-            SSL_free(ssl);
-            ap_ctx_set(fb->ctx, "ssl", NULL);
-            ap_bsetflag(fb, B_EOF|B_EOUT, 1);
-            conn->aborted = 1;
-            return;
-        }
-        if (SSL_use_RSAPrivateKey(ssl, sc->prsaKey) <= 0) {
-            ssl_log(conn->server, SSL_LOG_ERROR|SSL_ADD_SSLERR,
-                    "Unable to configure server private key for connection");
-            SSL_free(ssl);
-            ap_ctx_set(fb->ctx, "ssl", NULL);
-            ap_bsetflag(fb, B_EOF|B_EOUT, 1);
-            conn->aborted = 1;
-            return;
-        }
+    if (SSL_use_certificate(ssl, sc->px509Certificate) <= 0) {
+        ssl_log(conn->server, SSL_LOG_ERROR|SSL_ADD_SSLERR,
+                "Unable to configure server certificate for connection");
+        SSL_free(ssl);
+        ap_ctx_set(fb->ctx, "ssl", NULL);
+        ap_bsetflag(fb, B_EOF|B_EOUT, 1);
+        conn->aborted = 1;
+        return;
+    }
+    if (SSL_use_RSAPrivateKey(ssl, sc->prsaKey) <= 0) {
+        ssl_log(conn->server, SSL_LOG_ERROR|SSL_ADD_SSLERR,
+                "Unable to configure server private key for connection");
+        SSL_free(ssl);
+        ap_ctx_set(fb->ctx, "ssl", NULL);
+        ap_bsetflag(fb, B_EOF|B_EOUT, 1);
+        conn->aborted = 1;
+        return;
     }
 
     /*
@@ -252,16 +251,15 @@ void ssl_hook_NewConnection(conn_rec *conn)
             }
             else if (ERR_GET_REASON(ERR_peek_error()) == SSL_R_HTTP_REQUEST) {
                 /*
-                 * The case where SSLeay has recognized a HTTP request:
+                 * The case where OpenSSL has recognized a HTTP request:
                  * This means the client speaks plain HTTP on our HTTPS
                  * port. Hmmmm...  At least for this error we can be more friendly
                  * and try to provide him with a HTML error page. We have only one
-                 * problem: SSLeay has already read some bytes from the HTTP
+                 * problem: OpenSSL has already read some bytes from the HTTP
                  * request. So we have to skip the request line manually and
                  * instead provide a faked one in order to continue the internal
                  * Apache processing.
                  *
-                 * (This feature is only available for SSLeay 0.9.0 and higher)
                  */
                 char ca[2];
                 int rv;
@@ -366,6 +364,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
         if (sc->nVerifyClient != SSL_CVERIFY_NONE) {
             char *s;
             X509 *xs;
+
             if ((xs = SSL_get_peer_certificate(ssl)) != NULL) {
                 s = X509_NAME_oneline(X509_get_subject_name(xs), NULL, 0);
                 ap_ctx_set(fb->ctx, "ssl::client::dn", ap_pstrdup(conn->pool, s));
@@ -399,7 +398,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
 
     /*
      * Improve I/O throughput by using
-     * SSLeay's read-ahead functionality
+     * OpenSSL's read-ahead functionality
      * (don't used under Win32, because
      * there we use select())
      */
@@ -407,6 +406,12 @@ void ssl_hook_NewConnection(conn_rec *conn)
     SSL_set_read_ahead(ssl, TRUE);
 #endif
 
+#ifdef SSL_VENDOR
+    /* Allow vendors to do more things on connection time... */
+    ap_hook_use("ssl::vendor::new_connection", 
+                AP_HOOK_SIG2(void,ptr), AP_HOOK_ALL, conn);
+#endif
+
     return;
 }
 
@@ -423,9 +428,8 @@ void ssl_hook_TimeoutConnection(int sig)
 /*
  *  Close the SSL part of the socket connection
  */
-void ssl_hook_CloseConnection(void *_conn)
+void ssl_hook_CloseConnection(conn_rec *conn)
 {
-    conn_rec *conn = _conn;
     SSL *ssl;
 
     /*
@@ -433,7 +437,7 @@ void ssl_hook_CloseConnection(void *_conn)
      */
     ssl = ap_ctx_get(conn->client->ctx, "ssl");
     if (ssl != NULL) {
-        SSL_set_shutdown(ssl, SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
+        SSL_set_shutdown(ssl, SSL_RECEIVED_SHUTDOWN);
         while (!SSL_shutdown(ssl));
         SSL_free(ssl);
         ap_ctx_set(conn->client->ctx, "ssl", NULL);
@@ -542,7 +546,7 @@ int ssl_hook_Auth(request_rec *r)
     /*
      * Fake a password - which one would be immaterial, as, it seems, an empty
      * password in the users file would match ALL incoming passwords, if only
-     * we were using the standard crypt library routine. Unfortunately, SSLeay
+     * we were using the standard crypt library routine. Unfortunately, OpenSSL
      * "fixes" a "bug" in crypt and thus prevents blank passwords from
      * working.  (IMHO what they really fix is a bug in the users of the code
      * - failing to program correctly for shadow passwords).  We need,
@@ -566,7 +570,7 @@ int ssl_hook_Access(request_rec *r)
     SSLDirConfigRec *dc;
     SSLSrvConfigRec *sc;
     SSL *ssl;
-    SSL_CTX *ctx;
+    SSL_CTX *ctx = NULL;
     array_header *apRequirement;
     ssl_require_t *pRequirements;
     ssl_require_t *pRequirement;
@@ -585,6 +589,7 @@ int ssl_hook_Access(request_rec *r)
     int nVerifyOld;
     int nVerify;
     int n;
+    int rc;
 
     dc  = myDirConfig(r);
     sc  = mySrvConfig(r->server);
@@ -620,9 +625,9 @@ int ssl_hook_Access(request_rec *r)
      * Especially because while Apache is in its sub-request processing the
      * client could force additional handshakes, too. And these take place
      * perhaps without our notice. So the only possibility is to ask
-     * SSLeay/OpenSSL whether the renegotiation has to be performed or not. It
+     * OpenSSL whether the renegotiation has to be performed or not. It
      * has to performed when some parameters which were previously known (by
-     * us) are not those we've now reconfigured (as known by SSLeay/OpenSSL).
+     * us) are not those we've now reconfigured (as known by OpenSSL).
      */
     renegotiate = FALSE;
 #ifdef SSL_EXPERIMENTAL
@@ -669,7 +674,7 @@ int ssl_hook_Access(request_rec *r)
     /*
      * override of SSLVerifyDepth:
      * This is handled by us manually inside the verify callback
-     * function and not by SSLeay internally. And our function is
+     * function and not by OpenSSL internally. And our function is
      * aware of both the per-server and per-directory contexts.
      * All we have to do is to force the renegotiation when the
      * maximum allowed depth is changed.
@@ -725,7 +730,7 @@ int ssl_hook_Access(request_rec *r)
         /*
            FIXME: This should be...
            if (!SSL_load_verify_locations(ssl, cpCAFile, cpCAPath)) { 
-           ...but SSLeay/OpenSSL still doesn't provide this!
+           ...but OpenSSL still doesn't provide this!
          */
         if (!SSL_CTX_load_verify_locations(ctx, cpCAFile, cpCAPath)) {
             ssl_log(r->server, SSL_LOG_ERROR|SSL_ADD_SSLERR,
@@ -753,7 +758,7 @@ int ssl_hook_Access(request_rec *r)
         /* 
          * Now we force the SSL renegotation by sending the Hello Request
          * message to the client. Here we have to do a workaround: Actually
-         * SSLeay returns immediately after sending the Hello Request (the
+         * OpenSSL returns immediately after sending the Hello Request (the
          * intent AFAIK is because the SSL/TLS protocol says it's not a must
          * that the client replies to a Hello Request). But because we insist
          * on a reply (anything else is an error for us) we have to go to the
@@ -796,10 +801,10 @@ int ssl_hook_Access(request_rec *r)
     }
 
     /*
-     * Under old SSLeay we had to change the X509_STORE inside the SSL_CTX
-     * instead inside the SSL structure, so we have to reconfigure it to the
-     * old values. This should be changed with forthcoming OpenSSL version
-     * when better functionality is avaiable.
+	 * Under old OpenSSL we had to change the X509_STORE inside the
+	 * SSL_CTX instead inside the SSL structure, so we have to reconfigure it
+	 * to the old values. This should be changed with forthcoming OpenSSL
+	 * versions when better functionality is avaiable.
      */
 #ifdef SSL_EXPERIMENTAL
     if (renegotiate && reconfigured_locations) {
@@ -841,8 +846,15 @@ int ssl_hook_Access(request_rec *r)
 
     /*
      * Else access is granted...
+     * (except vendor handlers override)
      */
-    return OK;
+    rc = OK;
+#ifdef SSL_VENDOR
+    ap_hook_use("ssl::vendor::access_handler", 
+                AP_HOOK_SIG2(int,ptr), AP_HOOK_DECLINE(DECLINED), 
+                &rc, r);
+#endif
+    return rc;
 }
 
 /*
@@ -955,14 +967,18 @@ int ssl_hook_Fixup(request_rec *r)
 
 /*  _________________________________________________________________
 **
-**  SSLeay Callback Functions
+**  OpenSSL Callback Functions
 **  _________________________________________________________________
 */
 
 /*
  * Handle out the already generated RSA key...
  */
+#if SSL_LIBRARY_VERSION >= 0x0920
+RSA *ssl_callback_TmpRSA(SSL *pSSL, int nExport, int nKeyLen)
+#else
 RSA *ssl_callback_TmpRSA(SSL *pSSL, int nExport)
+#endif
 {
     SSLModConfigRec *mc = myModConfig();
 
@@ -970,7 +986,7 @@ RSA *ssl_callback_TmpRSA(SSL *pSSL, int nExport)
 }
 
 /*
- * This SSLeay callback function is called when SSLeay
+ * This OpenSSL callback function is called when OpenSSL
  * does client authentication and verifies the certificate chain.
  */
 int ssl_callback_SSLVerify(int ok, X509_STORE_CTX *ctx)
@@ -989,7 +1005,7 @@ int ssl_callback_SSLVerify(int ok, X509_STORE_CTX *ctx)
     int depth;
 
     /*
-     * Get Apache context back through SSLeay context
+     * Get Apache context back through OpenSSL context
      */
     ssl  = (SSL *)X509_STORE_CTX_get_app_data(ctx);
     conn = (conn_rec *)SSL_get_app_data(ssl);
@@ -1062,14 +1078,14 @@ int ssl_callback_SSLVerify(int ok, X509_STORE_CTX *ctx)
     }
 
     /*
-     * And finally signal SSLeay the (perhaps changed) state
+     * And finally signal OpenSSL the (perhaps changed) state
      */
     return (ok);
 }
 
 /*
- *  This callback function is executed by SSLeay whenever a new SSL_SESSION is
- *  added to the internal SSLeay session cache. We use this hook to spread the
+ *  This callback function is executed by OpenSSL whenever a new SSL_SESSION is
+ *  added to the internal OpenSSL session cache. We use this hook to spread the
  *  SSL_SESSION also to the inter-process disk-cache to make share it with our
  *  other Apache pre-forked server processes.
  */
@@ -1081,14 +1097,14 @@ int ssl_callback_NewSessionCacheEntry(SSL *ssl, SSL_SESSION *pNew)
     long t;
 
     /*
-     * Get Apache context back through SSLeay context
+     * Get Apache context back through OpenSSL context
      */
     conn = (conn_rec *)SSL_get_app_data(ssl);
     s    = conn->server;
     sc   = mySrvConfig(s);
 
     /*
-     * Set the timeout also for the internal SSLeay cache, because this way
+     * Set the timeout also for the internal OpenSSL cache, because this way
      * our inter-process cache is consulted only when it's really necessary.
      */
     t = (SSL_get_time(pNew) + sc->nSessionCacheTimeout);
@@ -1109,15 +1125,15 @@ int ssl_callback_NewSessionCacheEntry(SSL *ssl, SSL_SESSION *pNew)
             t-time(NULL));
 
     /*
-     * return 0 which means to SSLeay that the pNew is still
+     * return 0 which means to OpenSSL that the pNew is still
      * valid and was not freed by us with SSL_SESSION_free().
      */
     return 0;
 }
 
 /*
- *  This callback function is executed by SSLeay whenever a
- *  SSL_SESSION is looked up in the internal SSLeay cache and it
+ *  This callback function is executed by OpenSSL whenever a
+ *  SSL_SESSION is looked up in the internal OpenSSL cache and it
  *  was not found. We use this to lookup the SSL_SESSION in the
  *  inter-process disk-cache where it was perhaps stored by one
  *  of our other Apache pre-forked server processes.
@@ -1130,7 +1146,7 @@ SSL_SESSION *ssl_callback_GetSessionCacheEntry(
     SSL_SESSION *pSession;
 
     /*
-     * Get Apache context back through SSLeay context
+     * Get Apache context back through OpenSSL context
      */
     conn = (conn_rec *)SSL_get_app_data(ssl);
     s    = conn->server;
@@ -1163,8 +1179,8 @@ SSL_SESSION *ssl_callback_GetSessionCacheEntry(
 }
 
 /*
- *  This callback function is executed by SSLeay whenever a
- *  SSL_SESSION is removed from the the internal SSLeay cache.
+ *  This callback function is executed by OpenSSL whenever a
+ *  SSL_SESSION is removed from the the internal OpenSSL cache.
  *  We use this to remove the SSL_SESSION in the inter-process
  *  disk-cache, too.
  */
@@ -1174,7 +1190,7 @@ void ssl_callback_DelSessionCacheEntry(
     server_rec *s;
 
     /*
-     * Get Apache context back through SSLeay context
+     * Get Apache context back through OpenSSL context
      */
     s = (server_rec *)SSL_CTX_get_app_data(ctx);
 
@@ -1195,9 +1211,9 @@ void ssl_callback_DelSessionCacheEntry(
 }
 
 /*
- * This callback function is executed while SSLeay processes the
+ * This callback function is executed while OpenSSL processes the
  * SSL handshake and does SSL record layer stuff. We use it to
- * trace SSLeay's processing in out SSL logfile.
+ * trace OpenSSL's processing in out SSL logfile.
  */
 void ssl_callback_LogTracingState(SSL *ssl, int where, int rc)
 {
@@ -1234,7 +1250,8 @@ void ssl_callback_LogTracingState(SSL *ssl, int where, int rc)
                     SSL_LIBRARY_NAME, SSL_state_string_long(ssl));
         else if (where & SSL_CB_ALERT) {
             str = (where & SSL_CB_READ) ? "read" : "write";
-            ssl_log(s, SSL_LOG_TRACE, "%s: Alert: %s:%s\n", SSL_LIBRARY_NAME,
+            ssl_log(s, SSL_LOG_TRACE, "%s: Alert: %s:%s:%s\n", 
+                    SSL_LIBRARY_NAME, str,
                     SSL_alert_type_string_long(rc),
                     SSL_alert_desc_string_long(rc));
         }
