@@ -32,9 +32,22 @@
 #include <evp.h>
 #include <pem.h>
 
-#if SSLEAY_VERSION_NUMBER < 0x0900
+#ifdef OPENSSL_VERSION_NUMBER
+#define SSL_LIBRARY_VERSION OPENSSL_VERSION_NUMBER
+#else
+#ifdef SSLEAY_VERSION_NUMBER
+#define SSL_LIBRARY_VERSION SSLEAY_VERSION_NUMBER
+#else
+#define SSL_LIBRARY_VERSION 0x0000
+#endif
+#endif
+
+#if SSL_LIBRARY_VERSION < 0x0900
 #define OBJ_create(a,b,c) OBJ_create_and_add_object(a,b,c) 
 #endif
+/* #if SSL_LIBRARY_VERSION < 0x0920 */
+#define OpenSSL_add_all_algorithms SSLeay_add_all_algorithms
+/* #endif */
 
 typedef struct {
     int ca;
@@ -44,6 +57,7 @@ typedef struct {
 int i2d_BASIC_CONSTRAINTS(BASIC_CONSTRAINTS *, unsigned char **);
 ASN1_OBJECT *__OBJ_txt2obj(char *);
 
+#if SSL_LIBRARY_VERSION < 0x0920
 int i2d_BASIC_CONSTRAINTS(BASIC_CONSTRAINTS *a, unsigned char **pp)
 {
     M_ASN1_I2D_vars(a);
@@ -57,6 +71,7 @@ int i2d_BASIC_CONSTRAINTS(BASIC_CONSTRAINTS *a, unsigned char **pp)
     M_ASN1_I2D_put(a->pathlen, i2d_ASN1_INTEGER);
     M_ASN1_I2D_finish();
 }
+#endif
 
 typedef struct {
     char *name;
@@ -297,7 +312,7 @@ int main(int argc, char **argv)
         exit(1);
     }
 
-    SSLeay_add_all_algorithms();
+    OpenSSL_add_all_algorithms();
     X509v3_add_netscape_extensions();
     X509v3_add_standard_extensions();
 
@@ -527,10 +542,18 @@ int main(int argc, char **argv)
 
         extobj = __OBJ_txt2obj("2.5.29.37");
         /* generate encoding of extension */
+#if SSL_LIBRARY_VERSION >= 0x0920
+        extlen = i2d_ASN1_SET(extusage, NULL, i2d_ASN1_OBJECT, V_ASN1_SEQUENCE, V_ASN1_UNIVERSAL, IS_SET);
+#else
         extlen = i2d_ASN1_SET(extusage, NULL, i2d_ASN1_OBJECT, V_ASN1_SEQUENCE, V_ASN1_UNIVERSAL);
+#endif
         extder = malloc(extlen);
         p = extder;
+#if SSL_LIBRARY_VERSION >= 0x0920
+        i2d_ASN1_SET(extusage, &p, i2d_ASN1_OBJECT, V_ASN1_SEQUENCE, V_ASN1_UNIVERSAL, IS_SET);
+#else
         i2d_ASN1_SET(extusage, &p, i2d_ASN1_OBJECT, V_ASN1_SEQUENCE, V_ASN1_UNIVERSAL);
+#endif
         extkey = ASN1_OCTET_STRING_new();
         ASN1_OCTET_STRING_set(extkey, extder, extlen);
         free(extder);
